package oracle.retail.microapps.item.create.am;

import java.math.BigDecimal;

import java.sql.CallableStatement;
import java.sql.SQLException;
import java.sql.Types;

import java.util.Date;

import oracle.jbo.JboException;
import oracle.jbo.server.ApplicationModuleImpl;
import oracle.jbo.server.ViewObjectImpl;

import oracle.retail.microapps.item.create.am.common.CreateItem;
import oracle.retail.microapps.item.ribtypes.RibXitemdescRec;

import oracle.retail.microapps.item.ribtypes.RibXitemsupctydescRec;
import oracle.retail.microapps.item.ribtypes.RibXitemsupctydescTbl;
import oracle.retail.microapps.item.ribtypes.RibXitemsupdescRec;
import oracle.retail.microapps.item.ribtypes.RibXitemsupdescTbl;

import oracle.retail.microapps.item.ribtypes.RibXitmsupctymfrdescRec;
import oracle.retail.microapps.item.ribtypes.RibXitmsupctymfrdescTbl;

import org.apache.log4j.Logger;

public class CreateItemImpl extends ApplicationModuleImpl implements CreateItem {
    
    private Logger  logger = Logger.getLogger(CreateItemImpl.class);
    
    private String itemId; //Item Id created
    
    //parameters required to be set for item creation
    private String itemDesc; 
    private String unitCost;
    private String itemNumberManual;
    private String uomConversionFactor;
    private String supplierConfig; //supplier or supplierSite ?

    /**
     * This is the default constructor (do not remove).
     */
    public CreateItemImpl() {
    }

    /**
     * Container's getter for ClassVO1.
     * @return ClassVO1
     */
    public ViewObjectImpl getClassVO1() {
        return (ViewObjectImpl)findViewObject("ClassVO1");
    }

    /**
     * Container's getter for CountryVO1.
     * @return CountryVO1
     */
    public ViewObjectImpl getCountryVO1() {
        return (ViewObjectImpl)findViewObject("CountryVO1");
    }

    /**
     * Container's getter for DepartmentVO1.
     * @return DepartmentVO1
     */
    public ViewObjectImpl getDepartmentVO1() {
        return (ViewObjectImpl)findViewObject("DepartmentVO1");
    }

    /**
     * Container's getter for SubclassVO1.
     * @return SubclassVO1
     */
    public ViewObjectImpl getSubclassVO1() {
        return (ViewObjectImpl)findViewObject("SubclassVO1");
    }

    /**
     * Container's getter for SupplierSiteVO1.
     * @return SupplierSiteVO1
     */
    public ViewObjectImpl getSupplierSiteVO1() {
        return (ViewObjectImpl)findViewObject("SupplierSiteVO1");
    }

    /**
     * Container's getter for SupplierVO1.
     * @return SupplierVO1
     */
    public ViewObjectImpl getSupplierVO1() {
        return (ViewObjectImpl)findViewObject("SupplierVO1");
    }

    /**
     * Container's getter for ItemNumberTypeVO1.
     * @return ItemNumberTypeVO1
     */
    public ViewObjectImpl getItemNumberTypeVO1() {
        return (ViewObjectImpl)findViewObject("ItemNumberTypeVO1");
    }

    /**
     * Container's getter for CostZoneGroupVO1.
     * @return CostZoneGroupVO1
     */
    public ViewObjectImpl getCostZoneGroupVO1() {
        return (ViewObjectImpl)findViewObject("CostZoneGroupVO1");
    }

    /**
     * Container's getter for RetailPriceZoneGroupVO1.
     * @return RetailPriceZoneGroupVO1
     */
    public ViewObjectImpl getRetailPriceZoneGroupVO1() {
        return (ViewObjectImpl)findViewObject("RetailPriceZoneGroupVO1");
    }

    /**
     * Container's getter for StandardUomVO1.
     * @return StandardUomVO1
     */
    public ViewObjectImpl getStandardUomVO1() {
        return (ViewObjectImpl)findViewObject("StandardUomVO1");
    }

    /**
     * Container's getter for SystemOptionsVO1.
     * @return SystemOptionsVO1
     */
    public ViewObjectImpl getSystemOptionsVO1() {
        return (ViewObjectImpl)findViewObject("SystemOptionsVO1");
    }

    /**
     * This method calls a Business service to create an item in <b>RMS</b> schema.
     * The Business service used is a PL/SQL procedure <b>RMSSUB_XITEM.CONSUME</b> (subprocess : xitemcre)
     * defined in RMS schema.
     * <p>
     * This method is exposed to the clients through ADF's DataControl, from where it can be called by
     * any external java based system, typically a UI. Before calling this method client needs
     * to set the parameters required to create an item through the setter methods exposed alongside.
     * @return String -Status of the business service call as a formatted text.
     */
    public String createRegularItem(){
        logger.debug("*** Business service method 'createRegularItem' call in application module ***");
        CallableStatement callableStmt = null;
        String callableStatement = "begin "+"RMSSUB_XITEM.CONSUME(?,?,?,?); end;";
        try{
            callableStmt = getDBTransaction().createCallableStatement(callableStatement,0);
            callableStmt.registerOutParameter(1, Types.VARCHAR);
            callableStmt.registerOutParameter(2, Types.VARCHAR);
            callableStmt.setString(1, "");
            callableStmt.setString(2, "");
            callableStmt.setObject(3, getSqlObjForRegularItemCreation());
            callableStmt.setString(4, "xitemcre");
            logger.debug("Executing X API");
            callableStmt.executeUpdate();
            logger.debug("STATUS code : " + callableStmt.getString(1));
            logger.debug("ERROR if any : " + callableStmt.getString(2));
            String status = "";
            if(callableStmt.getString(1).equals("S")){
              status = "S";
            }else{
              status = callableStmt.getString(2);
            }
            getDBTransaction().commit();
            logger.debug("Changes commited ");
            return status;
        }catch(SQLException e){
            logger.error("Error in callable statement execution ", e);
            throw new JboException(e);
        }catch (RuntimeException re){
            logger.error("Runtime Exception in callable statement execution ", re);
            throw new JboException(re);
        }finally{
            try{
              logger.debug("Closing connection"); 
              callableStmt.close();
            }catch(SQLException e){
              logger.error("Error in callable statement closing ", e);
              throw new RuntimeException(e);
            }
        } 
    }
  
    /**
     * This method returns instance of the java representation of a RIB object <b>RibXitemdescRec</b> 
     * (RIB objects are defined as SQL type in RMS schema). This instance 
     * is populated with user supplied values, passed in Map object parameter.
     * @return RibXitemdescRec -Rib Object encapsulating user supplied values.
     * @throws SQLException
     */
    private RibXitemdescRec getSqlObjForRegularItemCreation() throws SQLException {
        RibXitemdescRec xitemdescRec = new RibXitemdescRec();
        String itemNumber = getNextItemNumber();
        logger.debug("Item number to be used : "+itemNumber);
        xitemdescRec.setItem(itemNumber); //item number
        
        xitemdescRec.setPackInd("N"); //packInd
        BigDecimal bigDec_1 = new BigDecimal(1); 
        xitemdescRec.setItemLevel(bigDec_1); //itemLevel
        xitemdescRec.setTranLevel(bigDec_1); //tranLevel
        xitemdescRec.setIsclocHierLevel("S"); //isclocHierLevel
        xitemdescRec.setIzpHierLevel("S"); //izpHierLevel
        xitemdescRec.setStoreOrdMult("E"); //storeOrdMult
        xitemdescRec.setForecastInd("N"); //forecastInd
        xitemdescRec.setSimplePackInd("N"); //simplePackInd
        xitemdescRec.setContainsInnerInd("N"); //containsInnerInd
        xitemdescRec.setSellableInd("Y"); //sellableInd
        xitemdescRec.setOrderableInd("Y"); //orderableInd
        xitemdescRec.setCreateDatetime(new java.sql.Timestamp(new Date().getTime())); //createDatetime
        xitemdescRec.setStatus("A"); //status
        xitemdescRec.setCatchWeightInd("N"); //catchWeightInd
        xitemdescRec.setConstDimenInd("N"); //constDimenInd
        xitemdescRec.setGiftWrapInd("N"); //giftWrapInd
        xitemdescRec.setShipAloneInd("N"); //shipAloneInd
        
        String deptId =  this.getDepartmentVO1().getCurrentRow().getAttribute("DepartmentId").toString();
        logger.debug("Department Id to be used : "+deptId);
        xitemdescRec.setDept(new BigDecimal(deptId)); //dept
        
        String classId = this.getClassVO1().getCurrentRow().getAttribute("ClassId").toString();
        logger.debug("Class Id to be used : "+classId);
        xitemdescRec.setClass_(new BigDecimal(classId)); //class
        
        String subclassId = this.getSubclassVO1().getCurrentRow().getAttribute("SubclassId").toString();
        logger.debug("Subclass Id to be used : "+subclassId);
        xitemdescRec.setSubclass(new BigDecimal(subclassId));  //subclass
        
        logger.debug("Item desc to be used : "+this.itemDesc);
        xitemdescRec.setItemDesc(this.itemDesc); //itemDesc
        xitemdescRec.setShortDesc(this.itemDesc);  //shortDesc
        
        String retailZoneGrpId = this.getRetailPriceZoneGroupVO1().getCurrentRow().getAttribute("RetailPriceZoneGrpId").toString();
        logger.debug("Retail Price Zone Grp Id to be used : "+retailZoneGrpId);
        xitemdescRec.setRetailZoneGroupId(new BigDecimal(retailZoneGrpId));    //retailZoneGroupId
        
        String costZoneGrpId = this.getCostZoneGroupVO1().getCurrentRow().getAttribute("ZoneGroupId").toString();
        logger.debug("Cost Zone Group Id to be used : "+costZoneGrpId);
        xitemdescRec.setCostZoneGroupId(new BigDecimal(costZoneGrpId));    //costZoneGroupId
        
        String standardUOM = this.getStandardUomVO1().getCurrentRow().getAttribute("UomId").toString();
        logger.debug("UOM to be used : "+standardUOM);
        xitemdescRec.setStandardUom(standardUOM);   //standardUom
        
        xitemdescRec.setXitemsupdescTbl(getXitemsupdescTbl());  //xitemsupdescTbl
        
        logger.debug("UOM conversion factor to be used : "+this.uomConversionFactor);
        xitemdescRec.setUomConvFactor(new BigDecimal(this.uomConversionFactor));//uomConvFactor
        
        String itemNoType = this.getItemNumberTypeVO1().getCurrentRow().getAttribute("ItemNoTypeCode").toString();
        logger.debug("Item number type to be used : "+itemNoType);
        xitemdescRec.setItemNumberType(itemNoType);  //itemNumberType
  
        return xitemdescRec;
    }
  
    private RibXitemsupdescTbl getXitemsupdescTbl() throws SQLException {
        RibXitemsupdescRec[] xitemsupdescRecArray = new RibXitemsupdescRec[]{getXitemsupdescRec()};
        RibXitemsupdescTbl xitemsupdescTbl = new RibXitemsupdescTbl(xitemsupdescRecArray);
        return xitemsupdescTbl;
    }
    
    private RibXitemsupdescRec getXitemsupdescRec() throws SQLException {
        RibXitemsupdescRec xitemsupdescRec = new RibXitemsupdescRec();
        String supplier = supplierConfig.equals("Supplier")
                ? this.getSupplierVO1().getCurrentRow().getAttribute("SupplierId").toString()
                : this.getSupplierSiteVO1().getCurrentRow().getAttribute("SupplierId").toString();
        logger.debug("Supplier to be used : "+supplier);
        xitemsupdescRec.setSupplier(supplier); 
        xitemsupdescRec.setPrimarySuppInd("Y");
        xitemsupdescRec.setXitemsupctydescTbl(getXitemsupctydescTbl());
        xitemsupdescRec.setDirectShipInd("N");
        xitemsupdescRec.setPalletName("PAL");
        xitemsupdescRec.setCaseName("CS");
        xitemsupdescRec.setInnerName("INR");
        xitemsupdescRec.setXitmsupctymfrdescTbl(getXitmsupctymfrdescTbl());
        return xitemsupdescRec;
    }
  
    private RibXitemsupctydescTbl getXitemsupctydescTbl() throws SQLException {
        RibXitemsupctydescRec[] xitemsupctydescRecArray = new RibXitemsupctydescRec[]{getXitemsupctydescRec()};
        RibXitemsupctydescTbl xitemsupctydescTbl = new RibXitemsupctydescTbl(xitemsupctydescRecArray);
        return xitemsupctydescTbl;
    }
    
    private RibXitmsupctymfrdescTbl getXitmsupctymfrdescTbl() throws SQLException {
        RibXitmsupctymfrdescRec[] xitmsupctymfrdescRecArray = new RibXitmsupctymfrdescRec[]{getXitmsupctymfrdescRec()};
        RibXitmsupctymfrdescTbl xitmsupctymfrdescTbl = new RibXitmsupctymfrdescTbl(xitmsupctymfrdescRecArray);
        return xitmsupctymfrdescTbl;
    }
    
    private RibXitemsupctydescRec getXitemsupctydescRec() throws SQLException {
        RibXitemsupctydescRec xitemsupctydescRec = new RibXitemsupctydescRec();
        
        String country = (String) this.getCountryVO1().getCurrentRow().getAttribute("CountryId");
        logger.debug("Country to be used : "+country);
        xitemsupctydescRec.setOriginCountryId(country);
        
        xitemsupctydescRec.setPrimaryCountryInd("Y");
        
        logger.debug("Unit cost to be used : "+this.unitCost);
        xitemsupctydescRec.setUnitCost(new BigDecimal(this.unitCost));
        
        BigDecimal bigDec_1 = new BigDecimal(1);
        xitemsupctydescRec.setSuppPackSize(bigDec_1);
        xitemsupctydescRec.setInnerPackSize(bigDec_1);
        xitemsupctydescRec.setTi(bigDec_1);
        xitemsupctydescRec.setHi(bigDec_1);
        return xitemsupctydescRec;
    }
    
    private RibXitmsupctymfrdescRec getXitmsupctymfrdescRec() throws SQLException {
        RibXitmsupctymfrdescRec xitmsupctymfrdescRec = new RibXitmsupctymfrdescRec();
        
        String country = this.getCountryVO1().getCurrentRow().getAttribute("CountryId").toString();
        logger.debug("Supplier manufacturing Country to be used : "+country);
        xitmsupctymfrdescRec.setManufacturerCtryId(country);
        
        xitmsupctymfrdescRec.setPrimaryManufacturerCtryInd("Y");
        return xitmsupctymfrdescRec;
    }
      
    /**
     * This method returns the Next item sequence to be used in RMS for the new item created.
     * If the Item number type is MANL (manual), the user supplied value is used as the next sequence
     * else helper method {@link #getNextItemNumber(String itemNumberType)} is called
     * to generate the next sequence in RMS.
     * @return String -Next sequence number for Item in RMS.
     */
    private String getNextItemNumber(){
      //if item number type is 'MANL' return the manual value entered by user
      //else return the value generated in RMS by procedure call.
      String itemNumberType = (String)this.getItemNumberTypeVO1().getCurrentRow().getAttribute("ItemNoTypeCode");
      String itemNumberSeq = (itemNumberType.equals("MANL")
          ? (this.itemNumberManual)
          : getNextItemNumber(itemNumberType));
      this.itemId = itemNumberSeq;
      return itemNumberSeq;
    }
  
    /**
     * This method generates sequence number for a new item in RMS.
     * It calls the PL/SQL procedure <b>NEXT_ITEM_NUMBER</b> in RMS to generate the sequence.
     * NOTE: 'NEXT_ITEM_NUMBER' is a custom PL/SQL procedure not present in RMS by default.
     * It acts as a wrapper function for calling <b>NEXT_ITEM_NUMBER_SQL.GET_NEXT</b> procedure in RMS.
     * @return String Sequence number generated for Item in RMS.
     */
    private String getNextItemNumber(String itemNumberType){
        logger.debug("Call for getting next item number");
        logger.debug("Item number type used is : "+itemNumberType);
        CallableStatement callableStmt = null;
        String callableStatement = "begin NEXT_ITEM_NUMBER(?,?,?); end;";  
        try{
            callableStmt = getDBTransaction().createCallableStatement(callableStatement,0);
            callableStmt.registerOutParameter(1, Types.VARCHAR);
            callableStmt.registerOutParameter(3, Types.VARCHAR);
            callableStmt.setString(1, "");
            callableStmt.setString(2, itemNumberType);
            callableStmt.setString(3, "");
            logger.debug("Executing Item seq generator PL/SQL procedure"); 
            callableStmt.executeUpdate();
            logger.debug("Item seq : " + callableStmt.getString(1)); 
            logger.debug("seq gen ERROR : " + callableStmt.getString(3));
            String itemSeq = callableStmt.getString(1);
            return itemSeq;
        }catch(SQLException e){
            logger.debug("Error: Check if the custom procedure NEXT_ITEM_NUMBER is present in RMS");
            logger.error("Error while generating next item sequence in RMS." ,e);
            throw new JboException(e);
        } finally{
            try{
              logger.debug("Closing connection"); 
              callableStmt.close();
            }catch(SQLException e){
              logger.error("Error while closing statement." ,e);
              throw new RuntimeException(e);
            }
        }    
    }

    public String getItemId() {
        return itemId;
    }

    public void setItemDesc(String itemDesc) {
        this.itemDesc = itemDesc;
    }

    public void setUnitCost(String unitCost) {
        this.unitCost = unitCost;
    }

    public void setItemNumberManual(String itemNumberManual) {
        this.itemNumberManual = itemNumberManual;
    }

    public void setUomConversionFactor(String uomConversionFactor) {
        this.uomConversionFactor = uomConversionFactor;
    }

    public void setSupplierConfig(String supplierConfig) {
        if(supplierConfig.equals("Supplier") || supplierConfig.equals("SupplierSite"))
            this.supplierConfig = supplierConfig;
        else
            throw new IllegalArgumentException("Allowed values : Supplier, SupplierSite");
    }
}
