package oracle.retail.microapps.item.transfer.am.applicationModule;

import java.util.Collection;
import java.util.List;

import oracle.adf.model.binding.DCIteratorBinding;

import oracle.adf.share.ADFContext;

import oracle.jbo.Key;
import oracle.jbo.NameValuePairs;
import oracle.jbo.Row;
import oracle.jbo.RowSetIterator;
import oracle.jbo.ViewObject;
import oracle.jbo.domain.Number;

import oracle.jbo.server.ViewObjectImpl;

import oracle.retail.merch20.common.ADFUtils;
import oracle.retail.microapps.item.transfer.am.common.TransferItem;
import oracle.retail.microapps.item.transfer.managed.*;

import oracle.retail.microapps.order.cancel.am.applicationModule.CancelOrderAMTest;

import org.apache.log4j.Logger;

import org.junit.*;
import static org.junit.Assert.*;

public class TransferItemAMTest {
    private TransferItem _amImpl = (TransferItem)TransferItemAMFixture.getInstance().getApplicationModule();
    private Number fromLoc;
    private Number toLoc;
    private String fromLoctype;
    private String toLoctype;
    private String item;
    private String itemDesc;
    private Number dept;
    private String avlQty;
    private ViewObject srcLocationVO;
    private ViewObject destLocationVO;
    private ViewObject deptVO;
    private ViewObject itemVO;
    private ViewObject transferItemVO;
    // get a logger instance
    Logger logger = Logger.getLogger(TransferItemAMTest.class);
    
    public TransferItemAMTest() {
    }

    @Before
    public void setUp() {  
        
      srcLocationVO = _amImpl.findViewObject("SrcLocationVO1");
      
      srcLocationVO.setNamedWhereClauseParam("to_loc", null);
      srcLocationVO.setNamedWhereClauseParam("to_loc_type", null);
      srcLocationVO.setNamedWhereClauseParam("ParamItem", null);
      logger.debug("Bind variables of SrcLocationVO set to null ");
      srcLocationVO.executeQuery();
      
      Row firstRow = srcLocationVO.first();
      fromLoc = (Number)firstRow.getAttribute("Location"); 
      fromLoctype = (String)firstRow.getAttribute("LocType");  
      logger.debug( "Data from src location : " + fromLoc+" " +fromLoctype);
      
      deptVO = _amImpl.findViewObject("DepartmentVO1");
      Row firstRow1 = deptVO.first();
      dept = (Number)firstRow1.getAttribute("Dept");   
      logger.debug("Attributes of DepartmentVO1 fetched" + dept);
      
      destLocationVO = _amImpl.findViewObject("DestLocationVO1");
      destLocationVO.setNamedWhereClauseParam("from_loc", fromLoc);
      destLocationVO.setNamedWhereClauseParam("from_loc_type", fromLoctype);
      destLocationVO.executeQuery();
      
      Row firstRow2 = destLocationVO.first();
      toLoc = (Number)firstRow2.getAttribute("Location");   
      toLoctype = (String)firstRow2.getAttribute("LocType");  
      logger.debug( toLoc + " " + toLoctype);
      
      itemVO = _amImpl.findViewObject("ItemVO1");
      itemVO.setNamedWhereClauseParam("from_loc", fromLoc);
      itemVO.setNamedWhereClauseParam("from_loc_type", fromLoctype);
      itemVO.setNamedWhereClauseParam("dept", null);
      itemVO.executeQuery();
      logger.debug("Bind variables values set for DestLocationVO1 and ItemVO1");  
    }

    @After
    public void tearDown() {
        fromLoc = null;
        toLoc = null;
        fromLoctype = null;
        toLoctype = null;
        dept = null;
        item = null;
        itemDesc = null;
        avlQty = null;
    }

    @Test
    public void testTransferItem() {
        Row firstRow3 = itemVO.first();
        item = (String)firstRow3.getAttribute("Item"); 
        logger.debug("Item : "+ item);
        itemDesc = (String)firstRow3.getAttribute("ItemDesc");  
        logger.debug("ItemDesc : "+ itemDesc);
        avlQty = firstRow3.getAttribute("StockOnHand").toString();
        logger.debug("Available qty : "+ avlQty);
        
        transferItemVO = _amImpl.findViewObject("TransferItemsVO1");
        logger.debug("TransferItemsVO1Iterator fetched");
        Row row = transferItemVO.createRow();
        logger.debug("Row created");
        row.setAttribute("itemId", item);
        logger.debug("Item set");
        row.setAttribute("itemDesc", itemDesc);
        logger.debug("itemDesc set");
        row.setAttribute("avlQty", avlQty);
        logger.debug("avlQty set");
        row.setAttribute("tsfQty", 2);
        transferItemVO.insertRow(row);
        logger.debug("rows set with attribute values");
      
        String status = _amImpl.transferItem();
        logger.debug("transferItem() method executed and status retrieved ");
        assertTrue(status.equals("S"));
        logger.debug("Execution of  transferItem test case completed");
    }
}
