package oracle.retail.microapps.map.config;

import java.lang.reflect.Field;

import java.util.List;


import org.apache.log4j.Logger;

import org.junit.Assert;
import org.junit.Before;
import static org.junit.Assert.*;
import org.junit.Test;

/**
 * This class is to test the  code segment to read Map configuration in package map.config
 * All test cases are dependent on the presence of sampleMapConfig.xml in config folder in
 * the project.
 */
public class MapConfigReaderTest {
    
    private Logger logger = Logger.getLogger(MapConfigReaderTest.class);
    private MapConfig mapConfig;
    
    public MapConfigReaderTest() {
        super();
    }
    
    @Before
    public void setUp(){
        Class mapConfigReaderClass = MapConfigReader.class;
        Field mapConfigFileNameField = null;
        try {
            mapConfigFileNameField = mapConfigReaderClass.getDeclaredField("MAP_CONFIG_FILE_NAME");
        } catch (NoSuchFieldException e) {
            Assert.fail("private field MAP_CONFIG_FILE_NAME not present");
        }
        mapConfigFileNameField.setAccessible(true);
        try {
            mapConfigFileNameField.set(null, "config/sampleMapConfig.xml");
        } catch (IllegalAccessException e) {
            logger.debug("Exception "+e);
            logger.debug("Set a private field. Its OK.");
        }
        mapConfig = MapConfigReader.readConfig();
    }
    
    @Test
    public void testMapConfigReader(){
        assertNotNull(mapConfig);
    }
    
    @Test
    public void testMapElementContents(){
        Assert.assertEquals(mapConfig.getStartingXCoordinate(), -88.33, 0.0);
        Assert.assertEquals(mapConfig.getStartingYCoordinate(), 40.54, 0.0);
        Assert.assertEquals(mapConfig.getZoomLevel(), 3.0, 0.0);
        MapConfig.Themes testThemesObj = mapConfig.getThemes();
        Assert.assertNotNull(testThemesObj);
        List<MapConfig.Themes.Theme> testThemeListObj = testThemesObj.getTheme();
        Assert.assertTrue(testThemeListObj.size() == 1);      
    }
    
    @Test
    public void testThemeElementContents(){
        MapConfig.Themes.Theme testThemeObj = mapConfig.getThemes().getTheme().get(0);
        Assert.assertEquals(testThemeObj.getId(), "TRAN_1");
        Assert.assertEquals(testThemeObj.getType(), "POINT_THEME");
        Assert.assertEquals(testThemeObj.getName(), "Transactions Theme");
        Assert.assertEquals(testThemeObj.getIteratorName(), "ItemTransactions1Iterator");
        Assert.assertEquals(testThemeObj.isRendered(), true);
        Assert.assertNotNull(testThemeObj.getBusinessDataMapping());
        Assert.assertNotNull(testThemeObj.getSpatialDataMapping());
        Assert.assertNotNull(testThemeObj.getLaunchableApps());
    }
    
    @Test
    public void testSpatialDataMappingElementContents(){
        MapConfig.Themes.Theme.SpatialDataMapping testSpatialDataMappingObj = mapConfig.getThemes().getTheme().get(0).getSpatialDataMapping();
        Assert.assertEquals(testSpatialDataMappingObj.getLatitude(), "Latitude");
        Assert.assertEquals(testSpatialDataMappingObj.getLongitude(), "Logitude");
        Assert.assertEquals(testSpatialDataMappingObj.getLabel(), "StoreName");
        Assert.assertEquals(testSpatialDataMappingObj.getKey(), "Location");
    }
    
    @Test
    public void testBusinessDataMappingElementContents(){
        MapConfig.Themes.Theme.BusinessDataMapping testBusinessDataMappingObj = mapConfig.getThemes().getTheme().get(0).getBusinessDataMapping();
        Assert.assertNotNull(testBusinessDataMappingObj.getData());
        Assert.assertTrue(testBusinessDataMappingObj.getData().size() == 1);
        MapConfig.Themes.Theme.BusinessDataMapping.Data testDataObj = testBusinessDataMappingObj.getData().get(0);
        Assert.assertEquals(testDataObj.getLabel(), "Item transactions");
        Assert.assertEquals(testDataObj.getIterAttr(), "TransactionsForItem");
        Assert.assertEquals(testDataObj.getFormat(), null);
    }
    
    @Test
    public void testLaunchableAppsElementContents(){
        MapConfig.Themes.Theme.LaunchableApps testLaunchableAppsObj = mapConfig.getThemes().getTheme().get(0).getLaunchableApps();
        Assert.assertNotNull(testLaunchableAppsObj);
        Assert.assertTrue(testLaunchableAppsObj.getApp().size() == 2);
        MapConfig.Themes.Theme.LaunchableApps.App testAppObj = testLaunchableAppsObj.getApp().get(1);
        Assert.assertEquals(testAppObj.getId(), "TRANSFER_ITEM");
        Assert.assertEquals(testAppObj.getName(), "Transfer Item");
        Assert.assertNotNull(testAppObj.getParameter());
    }
    
    @Test
    public void testParameterElementContents(){
        List<MapConfig.Themes.Theme.LaunchableApps.App.Parameter> testParameterListObj = 
            mapConfig.getThemes().getTheme().get(0).getLaunchableApps().getApp().get(1).getParameter();
        Assert.assertNotNull(testParameterListObj);
        Assert.assertTrue(testParameterListObj.size() == 1);
        MapConfig.Themes.Theme.LaunchableApps.App.Parameter testParameterObj = testParameterListObj.get(0);
        Assert.assertEquals(testParameterObj.getIterAttr(), "Location");
        Assert.assertEquals(testParameterObj.getDefaultValue(), "1231");
        Assert.assertEquals(testParameterObj.getDest(), "LOCATION");
    }
}
