package oracle.retail.microapps.vendor.update.am.applicationModule;

import oracle.jbo.JboException;
import oracle.jbo.Row;
import oracle.jbo.ViewObject;
import oracle.jbo.domain.Number;

import oracle.retail.microapps.vendor.update.am.common.UpdateVendor;

import org.junit.*;
import static org.junit.Assert.*;

public class UpdateVendorAMTest {
    private UpdateVendor _amImpl =
        (UpdateVendor)UpdateVendorAMFixture.getInstance().getApplicationModule();

    private Number supplierID;
    private String supName;
    private String contactName;
    private String contactPhone;
    private String contactEmail;
    private String commentDesc;
    private String terms;
    private String freightTerms;
    private String currencyCode;
    private ViewObject supplierDetailsVO;

    public UpdateVendorAMTest() {
    }

    @Before
    public void setUp() {
        ViewObject supplierVO = _amImpl.findViewObject("SupplierVO1");
        Row firstRow = supplierVO.first();
        supplierID = (Number)firstRow.getAttribute("Supplier");
        supplierDetailsVO = _amImpl.findViewObject("SupplierDetailsVO1");
        supplierDetailsVO.setNamedWhereClauseParam("supplierID", supplierID);
        supplierDetailsVO.executeQuery();
        Row firstDetailsRow = supplierDetailsVO.first();
        supName = (String)firstDetailsRow.getAttribute("SupName");
        contactName = (String)firstDetailsRow.getAttribute("ContactName");
        contactPhone = (String)firstDetailsRow.getAttribute("ContactPhone");
        contactEmail = (String)firstDetailsRow.getAttribute("CommentDesc");
        terms = (String)firstDetailsRow.getAttribute("Terms");
        freightTerms = (String)firstDetailsRow.getAttribute("FreightTerms");
        currencyCode = (String)firstDetailsRow.getAttribute("CurrencyCode");
    }

    @After
    public void tearDown() {
        supplierID = null;
        supName = null;
        contactPhone = null;
        contactEmail = null;
        terms = null;
        freightTerms = null;
        currencyCode = null;
    }

    @Test
    public void testUpdateVendorAllValidValues() {
        String localContactName = "Dexter";
        String localContactPhone = "512 512 5120";
        String localContactEmail = "dexter@cbs.com";
        String localCommentsDesc = "Dexter updated this supplier";
        Row first = supplierDetailsVO.first();

        first.setAttribute("ContactName", localContactName);
        first.setAttribute("ContactPhone", localContactPhone);
        first.setAttribute("ContactEmail", localContactEmail);
        first.setAttribute("CommentDesc", localCommentsDesc);

        String status = _amImpl.updateVendor();

        Row updatedFirst = supplierDetailsVO.first();
        String newContactName = (String)first.getAttribute("ContactName");
        String newContactPhone = (String)first.getAttribute("ContactPhone");
        String newContactEmail = (String)first.getAttribute("ContactEmail");
        String newComments = (String)first.getAttribute("CommentDesc");
        assertTrue(newContactName.equals(localContactName));
        assertTrue(newContactPhone.equals(localContactPhone));
        assertTrue(newContactEmail.equals(localContactEmail));
        assertTrue(newComments.equals(localCommentsDesc));
        assertTrue(status.equals("S"));
        revertAllChanges();

    }

    @Test(expected = JboException.class)
    public void testUpdateVendorWithInvalidContactName() {
        Row first = supplierDetailsVO.first();
        String invalidLocalContactName =
            "jdsf shjfhsdjfhsd sdhf dsfjh " + "sjdf hsjd hfkasfsd dsddf dshfgkdffh gkdfhgkdfs hgkjdshgksdhgkj" +
            "sdhghfsdhgjkdfshgkjfd hgk hdsf gkhdfghdew e hdksh gkjdshgkjdghdsf" +
            "hdf dskjhgjkdfhg iuerrdereghieruhferiuhfdsjkfhsdkjrhfaeuirhfre";
        first.setAttribute("ContactName", invalidLocalContactName);
    }

    @Test
    public void testUpdateVendorWithNullContactName() {
        Row first = supplierDetailsVO.first();
        String invalidLocalContactName = null;
        first.setAttribute("ContactName", invalidLocalContactName);
        String status = _amImpl.updateVendor();
        assertFalse(status.equals("S"));
    }

    @Test (expected = JboException.class)
    public void testUpdateVendorWithInvalidContactPhone() {
        Row first = supplierDetailsVO.first();
        String invalidLocalContactPhone = "1234567891010109837234672836283499";
        first.setAttribute("ContactPhone", invalidLocalContactPhone);
    }

    @Test 
    public void testUpdateVendorWithNullContactPhone() {
        Row first = supplierDetailsVO.first();
        String invalidLocalContactPhone = null;
        first.setAttribute("ContactPhone", invalidLocalContactPhone);
        String status = _amImpl.updateVendor();
        assertFalse(status.equals("S"));
    }

    @Test(expected = JboException.class)
    public void testUpdateVendorWithInvalidContactEmail() {
        Row first = supplierDetailsVO.first();
        String invalidLocalContactEmail =
            "iwe uiewr uew we rewtrewq riuertue" +
            "e iue ruyreiuter ytuierytuireyhtuieryhtuieuty  erkjt erterit eirutei " +
            "ireuti euteiuieru tewteut i;oweute";
        first.setAttribute("ContactEmail", invalidLocalContactEmail);
    }

    private void revertAllChanges() {
        Row first = supplierDetailsVO.first();
        first.setAttribute("ContactName", contactName);
        first.setAttribute("ContactPhone", contactPhone);
        first.setAttribute("ContactEmail", contactEmail);
        first.setAttribute("CommentDesc", commentDesc);
        _amImpl.updateVendor();
    }
}
