package oracle.retail.microapps.item.create.backing;

import java.util.HashMap;
import java.util.Map;

import javax.faces.context.FacesContext;
import javax.faces.event.ActionEvent;
import javax.faces.event.ValueChangeEvent;

import oracle.adf.share.ADFContext;
import oracle.adf.view.rich.component.rich.input.RichInputListOfValues;
import oracle.adf.view.rich.component.rich.input.RichInputText;
import oracle.adf.view.rich.component.rich.input.RichSelectBooleanCheckbox;
import oracle.adf.view.rich.component.rich.layout.RichPanelFormLayout;
import oracle.adf.view.rich.component.rich.layout.RichPanelGroupLayout;
import oracle.adf.view.rich.component.rich.layout.RichShowDetail;

import oracle.adf.view.rich.component.rich.nav.RichCommandButton;

import oracle.adf.view.rich.component.rich.output.RichOutputFormatted;

import oracle.adf.view.rich.component.rich.output.RichSeparator;
import oracle.adf.view.rich.component.rich.output.RichSpacer;

import oracle.binding.OperationBinding;

import oracle.jbo.ApplicationModule;

import oracle.retail.merch20.common.ADFUtils;

import org.apache.log4j.Logger;
import org.apache.myfaces.trinidad.component.UIXGroup;

public class CreateItem {
    
    private static Logger logger = Logger.getLogger(CreateItem.class);

    //mandatory input field components for creating item
    private RichInputText itemDesc;
    private RichInputListOfValues departmentId;
    private RichInputListOfValues classId;
    private RichInputListOfValues subclassId;
    private RichInputListOfValues supplierIdId;//supplier
    private RichInputListOfValues supplierId1Id;//supplier site
    private RichInputListOfValues countryIdId;
    private RichInputText unitCost;
    
    //advanced input field components for creating item
    private RichInputListOfValues itemNoTypeCodeId;
    private RichInputText itemNumber;
    private RichInputListOfValues zoneGroupIdId;
    private RichInputListOfValues retailPriceZoneGrpIdId;
    private RichInputListOfValues uomIdId;
    private RichInputText uomConversionFactor;
    
    //advanced fields default values
    private static String ITEM_NO_TYPE_CODE = "ITEM";
    private static String COST_ZONE_GROUP = "1000";
    private static String RETAIL_PRICE_ZONE = "1";
    private static String UNIT_OF_MEASURE = "EA";
    private static String UOM_CONVERSION_FACTOR = "1";
  
    //Status message to be displayed to the user
    private String statusMessage;
    
    //other UI components
    private RichPanelGroupLayout pgl1;
    private RichPanelFormLayout pfl1;
    private RichPanelFormLayout pfl2;
    private RichShowDetail sd1;
    private RichCommandButton cb1;
    private RichCommandButton cb2;
    private RichPanelFormLayout pfl3;
    private RichPanelGroupLayout pgl2;
    private RichPanelGroupLayout pgl3;
    private RichSpacer s1;
    private RichPanelFormLayout pfl4;
    private RichSelectBooleanCheckbox sbc1;
    private RichSeparator s2;
    private UIXGroup g1;
    private RichSpacer s3;
    private RichSpacer s4;
    private RichSpacer s5;
    private RichOutputFormatted ot1;


    public void setItemDesc(RichInputText it1) {
        this.itemDesc = it1;
    }
    
    public RichInputText getItemDesc() {
        return itemDesc;
    }

    public void setDepartmentId(RichInputListOfValues departmentId) {
        this.departmentId = departmentId;
    }

    public RichInputListOfValues getDepartmentId() {
        return departmentId;
    }

    public void setClassId(RichInputListOfValues classId) {
        this.classId = classId;
    }

    public RichInputListOfValues getClassId() {
        return classId;
    }

    public void setSubclassId(RichInputListOfValues subclassId) {
        this.subclassId = subclassId;
    }

    public RichInputListOfValues getSubclassId() {
        return subclassId;
    }

    public void setSupplierIdId(RichInputListOfValues supplierIdId) {
        this.supplierIdId = supplierIdId;
    }

    public RichInputListOfValues getSupplierIdId() {
        return supplierIdId;
    }

    public void setSupplierId1Id(RichInputListOfValues supplierId1Id) {
        this.supplierId1Id = supplierId1Id;
    }

    public RichInputListOfValues getSupplierId1Id() {
        return supplierId1Id;
    }

    public void setCountryIdId(RichInputListOfValues countryIdId) {
        this.countryIdId = countryIdId;
    }

    public RichInputListOfValues getCountryIdId() {
        return countryIdId;
    }
    
    public void setUnitCost(RichInputText it2) {
        this.unitCost = it2;
    }
    
    public RichInputText getUnitCost() {
        return unitCost;
    }
   
   /**
     * Gets the new value of Item desc and sets it in the model layer
     * by executing an operations binding.
     * @param valueChangeEvent
     */
    public void itemDescValueChangeListner(ValueChangeEvent valueChangeEvent) {
        logger.debug("In itemDescValueChangeListner ");
        String itemDesc = (valueChangeEvent.getNewValue() != null
                  ? valueChangeEvent.getNewValue().toString()
                  : null);
        logger.debug("Value got is "+itemDesc);
        if(valueChangeEvent.getOldValue() != valueChangeEvent.getNewValue()){
            OperationBinding ob = ADFUtils.getBindingContainer().getOperationBinding("setItemDesc");
            logger.debug("Setting item desc in the model layer");
            ob.getParamsMap().put("itemDesc", itemDesc);
            ob.execute();
        }
    }
    
    /**
     * Gets the new value of department. Sets the department value in the view scope,
     * to be used as bind variable in class and subclass view objects. Resets the
     * class and subclass fields. Sets the current row for department view object.
     * @param valueChangeEvent
     */
    public void deptIdValueChangeListner(ValueChangeEvent valueChangeEvent) {
        logger.debug("In deptIdValueChangeListner ");
        String deptId = (valueChangeEvent.getNewValue() != null
                  ? valueChangeEvent.getNewValue().toString()
                  : null);
        logger.debug("Value got is "+deptId);
        if(valueChangeEvent.getOldValue() != valueChangeEvent.getNewValue()){
            logger.debug("Setting current row in department vo with key : "+deptId);
            ADFUtils.findIterator("DepartmentVO1Iterator").setCurrentRowWithKeyValue(deptId);
            logger.debug("Setting dept ID in the view scope");
            ADFContext.getCurrent().getViewScope().put("deptId", new Integer(deptId));
            logger.debug("Executing ClassVO query");
            ADFUtils.findIterator("ClassVO1Iterator").executeQuery();
            logger.debug("Setting dependent class id to null");
            classId.setValue(null);
            logger.debug("Setting dependent subclass id to null");
            subclassId.setValue(null);
        }
    }
    
    /**
     * Gets the new value of class. Sets the class value in the view scope,
     * to be used as bind variable in subclass view objects. Resets the
     * subclass field.Sets the current row for class view object.
     * @param valueChangeEvent
     */
    public void classIdValueChangeListner(ValueChangeEvent valueChangeEvent) {
        logger.debug("In classIdValueChangeListner ");
        String classId = (valueChangeEvent.getNewValue() != null
                  ? valueChangeEvent.getNewValue().toString()
                  : null);
        logger.debug("Value got is "+classId);
        if(valueChangeEvent.getOldValue() != valueChangeEvent.getNewValue()){
            logger.debug("Setting current row in class vo with key : "+classId);
            ADFUtils.findIterator("ClassVO1Iterator").setCurrentRowWithKeyValue(classId);
            logger.debug("Setting Class ID in the view scope");
            ADFContext.getCurrent().getViewScope().put("classId", new Integer(classId));
            logger.debug("Executing SubclassVO query");
            ADFUtils.findIterator("SubclassVO1Iterator").executeQuery();
            logger.debug("Setting dependent subclass id to null");
            subclassId.setValue(null);
        }
    }
    
    /**
     * Gets the new value of subclass.Sets the current row for subclass view object.
     * @param valueChangeEvent
     */
    public void subclassIdValueChangeListner(ValueChangeEvent valueChangeEvent) {
        logger.debug("In subclassIdValueChangeListner ");
        String subclassId = (valueChangeEvent.getNewValue() != null
                  ? valueChangeEvent.getNewValue().toString()
                  : null);
        logger.debug("Value got is "+subclassId);
        if(valueChangeEvent.getOldValue() != valueChangeEvent.getNewValue()){
            logger.debug("Setting current row in subclass vo with key : "+subclassId);
            ADFUtils.findIterator("SubclassVO1Iterator").setCurrentRowWithKeyValue(subclassId);
        }
    }
    
    /**
     * Gets the new value of supplier. Sets the current row for supplier view object.
     * Indicates in the model layer that the RMS configuration supports Supplier.
     * @param valueChangeEvent
     */
    public void supplierIdValueChangeListner(ValueChangeEvent valueChangeEvent) {
        logger.debug("In supplierIdValueChangeListner ");
        String supplierId = (valueChangeEvent.getNewValue() != null
                  ? valueChangeEvent.getNewValue().toString()
                  : null);
        logger.debug("Value got is "+supplierId);
        //indicates in the model layer for supplier configuration. (as opposed to supplier site)
        OperationBinding ob = ADFUtils.getBindingContainer().getOperationBinding("setSupplierConfig");
        logger.debug("Setting Supplier as SupplierConfig in the model layer");
        ob.getParamsMap().put("supplierConfig", "Supplier");
        ob.execute();
        if(valueChangeEvent.getOldValue() != valueChangeEvent.getNewValue()){
            logger.debug("Setting current row in supplier vo with key : "+supplierId);
            ADFUtils.findIterator("SupplierVO1Iterator").setCurrentRowWithKeyValue(supplierId);
        }
    }
    
    /**
     * Gets the new value of supplierSite. Sets the current row for subclass view object.
     * Indicates in the model layer that the RMS configuration supports SupplierSite.
     * @param valueChangeEvent
     */
    public void supplierSiteIdValueChangeListner(ValueChangeEvent valueChangeEvent) {
        logger.debug("In supplierSiteIdValueChangeListner ");
        String supplierSiteId = (valueChangeEvent.getNewValue() != null
                  ? valueChangeEvent.getNewValue().toString()
                  : null);
        logger.debug("Value got is "+supplierSiteId);
        //indicates in the model layer for supplier site configuration. (as opposed to supplier)
        OperationBinding ob = ADFUtils.getBindingContainer().getOperationBinding("setSupplierConfig");
        logger.debug("Setting Supplier as SupplierConfig in the model layer");
        ob.getParamsMap().put("supplierConfig", "SupplierSite");
        ob.execute();
        if(valueChangeEvent.getOldValue() != valueChangeEvent.getNewValue()){
            logger.debug("Setting current row in supplierSite vo with key : "+supplierSiteId);
            ADFUtils.findIterator("SupplierSiteVO1Iterator").setCurrentRowWithKeyValue(supplierSiteId);
        }
    }
    
    /**
     * Gets the new value of country. Sets the current row for country view object.
     * @param valueChangeEvent
     */
    public void countryIdValueChangeListner(ValueChangeEvent valueChangeEvent) {
        logger.debug("In countryIdValueChangeListner ");
        String countryId = (valueChangeEvent.getNewValue() != null
                  ? valueChangeEvent.getNewValue().toString()
                  : null);
        logger.debug("Value got is "+countryId);
        if(valueChangeEvent.getOldValue() != valueChangeEvent.getNewValue()){
            logger.debug("Setting current row in country vo with key : "+countryId);
            ADFUtils.findIterator("CountryVO1Iterator").setCurrentRowWithKeyValue(countryId);
        }
    }
    
    /**
     * Gets the new value of unit cost and sets it in the model layer
     * by executing an operations binding.
     * @param valueChangeEvent
    */
    public void unitCostValueChangeListner(ValueChangeEvent valueChangeEvent) {
        logger.debug("In unitCostValueChangeListner ");
        String unitCost = (valueChangeEvent.getNewValue() != null
                  ? valueChangeEvent.getNewValue().toString()
                  : null);
        logger.debug("Value got is "+unitCost);
        if(valueChangeEvent.getOldValue() != valueChangeEvent.getNewValue()){
            OperationBinding ob = ADFUtils.getBindingContainer().getOperationBinding("setUnitCost");
            logger.debug("Setting unit cost in the model layer");
            ob.getParamsMap().put("unitCost", unitCost);
            ob.execute();
        }
    }
    
    /**
     * Advanced fields Check box value change listner. If selected, enables the 
     * advanced fields for item creation for user input.
     * @param valueChangeEvent
     */
    public void editAdvancedCheckBoxValueChangeListner(ValueChangeEvent valueChangeEvent){
        logger.debug("In editAdvancedCheckBoxValueChangeListner");
        if(sbc1.getValue() == Boolean.valueOf(true)){
            itemNoTypeCodeId.setDisabled(false);
            if(itemNoTypeCodeId.getValue().toString().equals("MANL")){
              itemNumber.setDisabled(false);
            }
            zoneGroupIdId.setDisabled(false);
            retailPriceZoneGrpIdId.setDisabled(false);
            uomIdId.setDisabled(false);
            uomConversionFactor.setDisabled(false);
        } else {
            itemNoTypeCodeId.setDisabled(true);
            itemNumber.setDisabled(true);
            zoneGroupIdId.setDisabled(true);
            retailPriceZoneGrpIdId.setDisabled(true);
            uomIdId.setDisabled(true);
            uomConversionFactor.setDisabled(true);
        }
    }
    
     /**
      * Gets the new value of item number type. Sets the current row for item number type view object.
      * @param valueChangeEvent
      */
    public void itemNoTypeCodeIdValueChangeListner(ValueChangeEvent valueChangeEvent){
        logger.debug("In itemNoTypeCodeIdValueChangeListner ");
        String itemNoTypeCode = (valueChangeEvent.getNewValue() != null
                  ? valueChangeEvent.getNewValue().toString()
                  : null);
        logger.debug("Value got is "+itemNoTypeCode);
        if(valueChangeEvent.getOldValue() != valueChangeEvent.getNewValue()){
            if(valueChangeEvent.getNewValue().equals("MANL")){
                logger.debug("Setting dependent manual item no field to null");
                itemNumber.setDisabled(false);
            } else {
                itemNumber.setDisabled(true);
            }
            itemNumber.setValue(null);
            logger.debug("Setting current row in item number type vo with key : "+itemNoTypeCode);
            ADFUtils.findIterator("ItemNumberTypeVO1Iterator").setCurrentRowWithKeyValue(itemNoTypeCode);
        }
    }
    
    /**
     * Gets the new value of manual item number and sets it in the model layer
     * by executing an operations binding.
     * @param valueChangeEvent
    */
    public void itemNumberValueChangeListner(ValueChangeEvent valueChangeEvent) {
        logger.debug("In itemNumberValueChangeListner ");
        String itemNumber = (valueChangeEvent.getNewValue() != null
                  ? valueChangeEvent.getNewValue().toString()
                  : null);
        logger.debug("Value got is "+itemNumber);
        if(valueChangeEvent.getOldValue() != valueChangeEvent.getNewValue()){
            OperationBinding ob = ADFUtils.getBindingContainer().getOperationBinding("setItemNumberManual");
            logger.debug("Setting item number manual in the model layer");
            ob.getParamsMap().put("itemNumberManual", itemNumber);
            ob.execute();
        }
    }
  
    /**
     * Gets the new value of cost zone value. Sets the current row for cost zone view object.
     * @param valueChangeEvent
     */
    public void costZoneValueChangeListner(ValueChangeEvent valueChangeEvent) {
        logger.debug("In costZoneValueChangeListner ");
        String costZone = (valueChangeEvent.getNewValue() != null
                  ? valueChangeEvent.getNewValue().toString()
                  : null);
        logger.debug("Value got is "+costZone);
        if(valueChangeEvent.getOldValue() != valueChangeEvent.getNewValue()){
            logger.debug("Setting current row in cost zone vo with key : "+costZone);
            ADFUtils.findIterator("CostZoneGroupVO1Iterator").setCurrentRowWithKeyValue(costZone);
        }
    }
    
    /**
     * Gets the new value of retail zone value. Sets the current row for retail zone view object.
     * @param valueChangeEvent
     */
    public void retailZoneValueChangeListner(ValueChangeEvent valueChangeEvent) {
        logger.debug("In retailZoneValueChangeListner ");
        String retailZone = (valueChangeEvent.getNewValue() != null
                  ? valueChangeEvent.getNewValue().toString()
                  : null);
        logger.debug("Value got is "+retailZone);
        if(valueChangeEvent.getOldValue() != valueChangeEvent.getNewValue()){
            logger.debug("Setting current row in retail zone vo with key : "+retailZone);
            ADFUtils.findIterator("RetailPriceZoneGroupVO1Iterator").setCurrentRowWithKeyValue(retailZone);
        }
    }
    
    /**
     * Gets the new value of UOM value. Sets the current row for UOM view object.
     * @param valueChangeEvent
     */
    public void uomValueChangeListner(ValueChangeEvent valueChangeEvent) {
        logger.debug("In uomValueChangeListner ");
        String uom = (valueChangeEvent.getNewValue() != null
                  ? valueChangeEvent.getNewValue().toString()
                  : null);
        logger.debug("Value got is "+uom);
        if(valueChangeEvent.getOldValue() != valueChangeEvent.getNewValue()){
            logger.debug("Setting current row in unit of measure vo with key : "+uom);
            ADFUtils.findIterator("StandardUomVO1Iterator").setCurrentRowWithKeyValue(uom);
        }
    }
    
    /**
     * Gets the new value of UOM conversion factor and sets it in the model layer
     * by executing an operations binding.
     * @param valueChangeEvent
    */
    public void uomConversionFactorValueChangeListner(ValueChangeEvent valueChangeEvent) {
        logger.debug("In uomConversionFactorValueChangeListner ");
        String uomConvFactor = (valueChangeEvent.getNewValue() != null
                  ? valueChangeEvent.getNewValue().toString()
                  : null);
        logger.debug("Value got is "+uomConvFactor);
        if(valueChangeEvent.getOldValue() != valueChangeEvent.getNewValue()){
            OperationBinding ob = ADFUtils.getBindingContainer().getOperationBinding("setUomConversionFactor");
            logger.debug("Setting UOM conversion factor in the model layer");
            ob.getParamsMap().put("uomConversionFactor", uomConvFactor);
            ob.execute();
        }
    }
    
    /**
     * Call the business service for creating an item in RMS. The business service is 
     * exposed in the application module in the model layer.
     */
    public void createItem(ActionEvent e){
        
      ApplicationModule am = ADFUtils.getApplicationModuleForDataControl("CreateItemDataControl");
      logger.debug("Fetching the instance of Application Module");
      oracle.retail.microapps.item.create.am.common.CreateItem create = 
          (oracle.retail.microapps.item.create.am.common.CreateItem)am;
      logger.debug("Calling transferItem() method of Application Module");
      String result = create.createRegularItem();

      logger.debug("Calling createItem() method of Application Module");
      if (result.equals("S")) {
          String itemNo = create.getItemId();

         result = "Item No (" + itemNo + ") has been created successfully";
          logger.debug("result" + result);
      } else {
          result = "<BIG>ERROR</BIG> - " + result;  
      }
      setStatusMessage(result);
    }
  
    public void setStatusMessage(String statusMessage) {
        this.statusMessage = statusMessage;
    }
  
    public String getStatusMessage() {
        return statusMessage;
    }
  
    /**
     * Sets item number type code to a default value. Set the corresponding view object's
     * current row to default.
     * @param itemNoTypeCodeId
     */
    public void setItemNoTypeCodeId(RichInputListOfValues itemNoTypeCodeId) {
        this.itemNoTypeCodeId = itemNoTypeCodeId;
        if(this.itemNoTypeCodeId.getValue() == null){
            logger.debug("Setting default value for itemNoTypeCodeId as "+ITEM_NO_TYPE_CODE);
            this.itemNoTypeCodeId.setValue(ITEM_NO_TYPE_CODE);
            logger.debug("Setting current row for item no type code VO as "+ITEM_NO_TYPE_CODE);
            ADFUtils.findIterator("ItemNumberTypeVO1Iterator").setCurrentRowWithKeyValue(ITEM_NO_TYPE_CODE);
        }
    }

    public RichInputListOfValues getItemNoTypeCodeId() {
        return itemNoTypeCodeId;
    }

    /**
     * Sets the cost zone group to a default value. Set the corresponding view object's
     * current row to default.
     * @param zoneGroupIdId
     */
    public void setZoneGroupIdId(RichInputListOfValues zoneGroupIdId) {
        this.zoneGroupIdId = zoneGroupIdId;
        if(this.zoneGroupIdId.getValue() == null){
            logger.debug("Setting default value for zoneGroupIdId as "+COST_ZONE_GROUP);
            this.zoneGroupIdId.setValue(COST_ZONE_GROUP);
            logger.debug("Setting current row for zoneGroupId VO as "+COST_ZONE_GROUP);
            ADFUtils.findIterator("CostZoneGroupVO1Iterator").setCurrentRowWithKeyValue(COST_ZONE_GROUP);
        }
    }

    public RichInputListOfValues getZoneGroupIdId() {
        return zoneGroupIdId;
    }

    /**
     * Sets the standard UOM to a default value. Set the corresponding view object's
     * current row to default.
     * @param uomIdId
     */
    public void setUomIdId(RichInputListOfValues uomIdId) {
        this.uomIdId = uomIdId;
        if(this.uomIdId.getValue() == null){
            logger.debug("Setting default value for uomId as "+UNIT_OF_MEASURE);
            this.uomIdId.setValue(UNIT_OF_MEASURE);
            logger.debug("Setting current row for uomId VO as "+UNIT_OF_MEASURE);
            ADFUtils.findIterator("StandardUomVO1Iterator").setCurrentRowWithKeyValue(UNIT_OF_MEASURE);
        }
    }

    public RichInputListOfValues getUomIdId() {
        return uomIdId;
    }

    /**
     * Sets the retail price zone group to a default value. Set the corresponding view object's
     * current row to default.
     * @param retailPriceZoneGrpIdId
     */
    public void setRetailPriceZoneGrpIdId(RichInputListOfValues retailPriceZoneGrpIdId) {
        this.retailPriceZoneGrpIdId = retailPriceZoneGrpIdId;
        if(this.retailPriceZoneGrpIdId.getValue() == null){
            logger.debug("Setting default value for retailPriceZoneGrpId as "+RETAIL_PRICE_ZONE);
            this.retailPriceZoneGrpIdId.setValue(RETAIL_PRICE_ZONE);
            logger.debug("Setting current row for retailPriceZoneGrpId VO as "+RETAIL_PRICE_ZONE);
            ADFUtils.findIterator("RetailPriceZoneGroupVO1Iterator").setCurrentRowWithKeyValue(RETAIL_PRICE_ZONE);
        }
    }

    public RichInputListOfValues getRetailPriceZoneGrpIdId() {
        return retailPriceZoneGrpIdId;
    }
    
    /**
     * Sets the UOM conversion factor to a default value. Set the default value
     * in the model layer by executing an operations binding.
     * @param it4
     */
    public void setUomConversionFactor(RichInputText it4) {
        this.uomConversionFactor = it4;
        if(this.uomConversionFactor.getValue() == null){
            logger.debug("Setting default value for UOM conversion factor as "+UOM_CONVERSION_FACTOR );
            this.uomConversionFactor.setValue(UOM_CONVERSION_FACTOR);
            OperationBinding ob = ADFUtils.getBindingContainer().getOperationBinding("setUomConversionFactor");
            logger.debug("Setting UOM conversion factor in the model layer");
            ob.getParamsMap().put("uomConversionFactor", UOM_CONVERSION_FACTOR);
            ob.execute();
        }
    }
  
    public RichInputText getUomConversionFactor() {
        return uomConversionFactor;
    }
    
    public void cancel(ActionEvent e){
        logger.debug("Cancel action called");
        //disabling advanced fields
        sbc1.setValue(false);
        editAdvancedCheckBoxValueChangeListner(null);
        //resetting default values
        itemNoTypeCodeId.setValue(ITEM_NO_TYPE_CODE);
        ADFUtils.findIterator("ItemNumberTypeVO1Iterator").setCurrentRowWithKeyValue(ITEM_NO_TYPE_CODE);
        zoneGroupIdId.setValue(COST_ZONE_GROUP);
        ADFUtils.findIterator("CostZoneGroupVO1Iterator").setCurrentRowWithKeyValue(COST_ZONE_GROUP);
        retailPriceZoneGrpIdId.setValue(RETAIL_PRICE_ZONE);
        ADFUtils.findIterator("RetailPriceZoneGroupVO1Iterator").setCurrentRowWithKeyValue(RETAIL_PRICE_ZONE);
        uomIdId.setValue(UNIT_OF_MEASURE);
        ADFUtils.findIterator("StandardUomVO1Iterator").setCurrentRowWithKeyValue(UNIT_OF_MEASURE);
        uomConversionFactor.setValue(UOM_CONVERSION_FACTOR);
        OperationBinding ob = ADFUtils.getBindingContainer().getOperationBinding("setUomConversionFactor");
        logger.debug("Setting UOM conversion factor in the model layer");
        ob.getParamsMap().put("uomConversionFactor", UOM_CONVERSION_FACTOR);
        ob.execute();
    }
    
    public void setCb1(RichCommandButton cb1) {
        this.cb1 = cb1;
    }
  
    public RichCommandButton getCb1() {
        return cb1;
    }
  
    public void setCb2(RichCommandButton cb2) {
        this.cb2 = cb2;
    }
  
    public RichCommandButton getCb2() {
        return cb2;
    }
    
    public void setPfl3(RichPanelFormLayout pfl3) {
        this.pfl3 = pfl3;
    }
  
    public RichPanelFormLayout getPfl3() {
        return pfl3;
    }
  
    public void setPgl2(RichPanelGroupLayout pgl2) {
        this.pgl2 = pgl2;
    }
  
    public RichPanelGroupLayout getPgl2() {
        return pgl2;
    }
  
    public void setPgl3(RichPanelGroupLayout pgl3) {
        this.pgl3 = pgl3;
    }
  
    public RichPanelGroupLayout getPgl3() {
        return pgl3;
    }


    public void setS1(RichSpacer s1) {
        this.s1 = s1;
    }
  
    public RichSpacer getS1() {
        return s1;
    }
  
    public void setPfl4(RichPanelFormLayout pfl4) {
        this.pfl4 = pfl4;
    }
  
    public RichPanelFormLayout getPfl4() {
        return pfl4;
    }
  
    public void setSbc1(RichSelectBooleanCheckbox sbc1) {
        this.sbc1 = sbc1;
    }
  
    public RichSelectBooleanCheckbox getSbc1() {
        return sbc1;
    }
  
    public void setPgl1(RichPanelGroupLayout pgl1) {
        this.pgl1 = pgl1;
    }
    
    public RichPanelGroupLayout getPgl1() {
        return pgl1;
    }
    
    public void setPfl1(RichPanelFormLayout pfl1) {
        this.pfl1 = pfl1;
    }
    
    public RichPanelFormLayout getPfl1() {
        return pfl1;
    }
    
    public void setPfl2(RichPanelFormLayout pfl2) {
        this.pfl2 = pfl2;
    }
    
    public RichPanelFormLayout getPfl2() {
        return pfl2;
    }
    
    public void setSd1(RichShowDetail sd1) {
        this.sd1 = sd1;
    }
    
    public RichShowDetail getSd1() {
        return sd1;
    }
  
    public void setItemNumber(RichInputText it3) {
        this.itemNumber = it3;
    }
  
    public RichInputText getItemNumber() {
        return itemNumber;
    }
    
    public void setS2(RichSeparator s2) {
        this.s2 = s2;
    }
  
    public RichSeparator getS2() {
        return s2;
    }

    public void setG1(UIXGroup g1) {
        this.g1 = g1;
    }

    public UIXGroup getG1() {
        return g1;
    }

    public void setS3(RichSpacer s3) {
        this.s3 = s3;
    }

    public RichSpacer getS3() {
        return s3;
    }

    public void setS4(RichSpacer s4) {
        this.s4 = s4;
    }

    public RichSpacer getS4() {
        return s4;
    }

    public void setS5(RichSpacer s5) {
        this.s5 = s5;
    }

    public RichSpacer getS5() {
        return s5;
    }

    public void setOt1(RichOutputFormatted ot1) {
        this.ot1 = ot1;
    }

    public RichOutputFormatted getOt1() {
        return ot1;
    }
}
