package oracle.retail.microapps.item.transfer.managed;

import oracle.adf.model.binding.DCIteratorBinding;
import oracle.adf.share.ADFContext;

import oracle.jbo.Row;

import oracle.retail.merch20.common.ADFUtils;

import oracle.retail.microapps.item.transfer.backing.TransferItem;

import org.apache.log4j.Logger;

public class Transfer {
  
    //getting logger instance
    private Logger logger = Logger.getLogger(Transfer.class);
    //input params accepted by the transferItem view activity
    private String srcLocationInputParam; //source location
    private String srcLocationTypeInputParam; // source location type
    private String destLocationInputParam; // destination Location
    private String destLocationTypeInputParam; // destination location type
    private String itemInputParam; // transfer Item 
    public Transfer() {
        super();
    }

    public void setSrcLocationInputParam(String srcLocationInputParam) {
        this.srcLocationInputParam = srcLocationInputParam;
        logger.debug("In TransferBean : Source Location parameter passed is: " + srcLocationInputParam);
        DCIteratorBinding dcib = null;
        Row rw = null;
        if(srcLocationInputParam != null){
            dcib = ADFUtils.findIterator("SrcLocationVO1Iterator");
            dcib.setCurrentRowWithKeyValue(srcLocationInputParam);
            rw = dcib.getCurrentRow();
            logger.debug("Current row is : "+rw);
        }
        if (rw != null){
            //source location passed is valid
            //Fetching all the variables from location current row
            Object locationIdValue = rw.getAttribute("Location");
            Object locationTypeValue = rw.getAttribute("LocType");
            logger.debug("Source Location fetched from srcLocation vo is " + locationIdValue);
            logger.debug("Source Location type value fetched from srcLocation vo is " + locationTypeValue);
            //setting view scope variables for queries
            ADFContext.getCurrent().getViewScope().put("fromLocationBindValue", locationIdValue.toString());
            ADFContext.getCurrent().getViewScope().put("fromLocTypeBindValue", locationTypeValue.toString());
        } else {
            //if location passed is not present
            logger.debug("Invalid Source location passed.");
            ADFContext.getCurrent().getViewScope().put("fromLocationBindValue", null);
            ADFContext.getCurrent().getViewScope().put("fromLocTypeBindValue", null);
        }
    }

    public String getSrcLocationInputParam() {
        return srcLocationInputParam;
    }

    public void setDestLocationInputParam(String destLocationInputParam) {
        this.destLocationInputParam = destLocationInputParam;
        logger.debug("In TransferBean : Destination Location parameter passed is: " + destLocationInputParam);
        DCIteratorBinding dcib = null;
        Row rw = null;
        if(destLocationInputParam != null){
            dcib = ADFUtils.findIterator("DestLocationVO1Iterator");
            dcib.setCurrentRowWithKeyValue(destLocationInputParam);
            rw = dcib.getCurrentRow();
        }
        if (rw != null){
            //destination location passed is valid
            //Fetching all the variables from location current row
            Object locationIdValue = rw.getAttribute("Location");
            Object locationTypeValue = rw.getAttribute("LocType");
            logger.debug("Destination Location fetched from destLocation vo is " + locationIdValue);
            logger.debug("Destination Location type value fetched from destLocation vo is " + locationTypeValue);
            //setting view scope variables for queries
            ADFContext.getCurrent().getViewScope().put("toLocBindValue", locationIdValue.toString());
            ADFContext.getCurrent().getViewScope().put("toLocTypeBindValue", locationTypeValue.toString());
        } else {
            //if location passed is not present
            logger.debug("Invalid destination location passed.");
            ADFContext.getCurrent().getViewScope().put("toLocBindValue", null);
            ADFContext.getCurrent().getViewScope().put("toLocTypeBindValue", null);
        }
    }

    public String getDestLocationInputParam() {
        return destLocationInputParam;
    }

    public void setItemInputParam(String itemInputParam) {
        this.itemInputParam = itemInputParam;
        logger.debug("In TransferBean : Item parameter passed is : "+itemInputParam);
        DCIteratorBinding dcib = null;
        Row rw = null;
        if(itemInputParam != null){
            dcib = ADFUtils.findIterator("ItemVO1Iterator"); 
            dcib.setCurrentRowWithKeyValue(itemInputParam);
            rw = dcib.getCurrentRow();
        }
        if(rw != null){
            //item passed is valid
            //create a row in transfer item view object
            TransferItem.populateTransferItemVO(rw);
            ADFContext.getCurrent().getViewScope().put("itemBindValue", itemInputParam);
        } else {
            logger.debug("Invalid item passed.");
            ADFContext.getCurrent().getViewScope().put("itemBindValue", null);
        }
    }

    public String getItemInputParam() {
        return itemInputParam;
    }

    public void setSrcLocationTypeInputParam(String srcLocationTypeInputParam) {
        this.srcLocationTypeInputParam = srcLocationTypeInputParam;
        logger.debug("In TransferBean : Source Location Type parameter passed is: " + srcLocationTypeInputParam);
        if(this.srcLocationTypeInputParam != null)
          ADFContext.getCurrent().getViewScope().put("fromLocTypeBindValue", srcLocationTypeInputParam.toString());
    }

    public String getSrcLocationTypeInputParam() {
        return srcLocationTypeInputParam;
    }

    public void setDestLocationTypeInputParam(String destLocationTypeInputParam) {
        this.destLocationTypeInputParam = destLocationTypeInputParam;
        logger.debug("In TransferBean : Destination Location Type parameter passed is: " + destLocationTypeInputParam);
        if(this.destLocationTypeInputParam != null)
          ADFContext.getCurrent().getViewScope().put("toLocTypeBindValue", destLocationTypeInputParam.toString());
        
    }

    public String getDestLocationTypeInputParam() {
        return destLocationTypeInputParam;
    }
}
