package oracle.retail.microapps.map.managed.helper;

/**
 * The Map application has the ability to launch dynamic taskflows when 
 * right clicked at a data point in a Map theme. This class store the 
 * dynamic task flow IDs for all such applications available.
 */
public class InContextAppDispatcher {
    
    /**
     * Dynamic task flow ID for Create Item microTask.
     */
    private static final String CREATE_ITEM = "/WEB-INF/create-item-flow.xml#create-item-flow";
    
    /**
     * Dynamic task flow ID for Transfer Item microTask.
     */
    private static final String TRANSFER_ITEM = "/WEB-INF/transfer-item-flow.xml#transfer-item-flow";
    
    /**
     * Dynamic task flow ID for Order Item microTask.
     */
    private static final String ORDER_ITEM = "/WEB-INF/create-order-flow.xml#create-order-flow";
    
    private InContextAppDispatcher() {
        super();
    }
    
    /**
     * @param appID (String) the ID of the application to be launched 
     * in a dynamic task flow.
     * @return (String) The dynamic task flow ID for the application.
     */
    public static String getAppTaskFlowID(String appID){
        if(appID.equals("CREATE_ITEM")){
            return CREATE_ITEM;
        } else if(appID.equals("TRANSFER_ITEM")){
            return TRANSFER_ITEM;
        } else if(appID.equals("ORDER_ITEM")){
            return ORDER_ITEM;
        } else {
            return "";
        }
    }
}
