package oracle.retail.microapps.order.cancel.am;

import java.math.BigDecimal;

import java.sql.CallableStatement;
import java.sql.SQLException;
import java.sql.Types;

import oracle.jbo.JboException;
import oracle.jbo.server.ApplicationModuleImpl;
import oracle.jbo.server.ViewObjectImpl;

import oracle.retail.microapps.order.cancel.am.common.CancelOrder;
import oracle.retail.microapps.order.ribtypes.RibXorderdtlRec;
import oracle.retail.microapps.order.ribtypes.RibXorderdescRec;
import oracle.retail.microapps.order.ribtypes.RibXorderdtlTbl;

import org.apache.log4j.Logger;
// ---------------------------------------------------------------------
// ---    File generated by Oracle ADF Business Components Design Time.
// ---    Mon Oct 04 10:45:36 IST 2010
// ---    Custom code may be added to this class.
// ---    Warning: Do not modify method signatures of generated methods.
// ---------------------------------------------------------------------
/**
 * CancelOrderImpl class implements the default ApplicationModule. This provides access to
 * all the View Objects used in the Application Module. This also contains the business
 * logic to perform all the necessary CRUD operations.
 *
 * @author Sachin Gupta, Ritesh Suresh
 *
 * @version 1.0
 *
 * @since 4th October 2010
 */
public class CancelOrderImpl extends ApplicationModuleImpl implements CancelOrder {
    //Create a Log Instance
    Logger logger = Logger.getLogger(CancelOrderImpl.class);

    /**
     * This is the default constructor (do not remove).
     */
    public CancelOrderImpl() {
    }

    /**
     * Container's getter for OrderLocVO1.
     * @return OrderLocVO1
     */
    public ViewObjectImpl getOrderLocVO1() {
        return (ViewObjectImpl)findViewObject("OrderLocVO1");
    }
  
    /**
     * Container's getter for SelectOrderVO1.
     * @return SelectOrderVO1
     */
    public ViewObjectImpl getSelectOrderVO1() {
        return (ViewObjectImpl)findViewObject("SelectOrderVO1");
    }

    /**
     * This method is exposed to the client via data control. This is used to
     * perform the Cancel Order operation. This method consumes RMSSUB_XORDER
     * package. The currently selected orderNo is fetched from the current row in
     * the SelectOrderVO1 View Object.
     * @return String - Returns the status of the cancel order operation.
     */
    public String cancelOrder() {
        logger.debug("Method cancelOrder in CancelOrderImpl called");
        String status = "";
        CallableStatement callableStmt = null;
       
        String callableStatement = "begin "  + "RMSSUB_XORDER.CONSUME(?,?,?,?); end;";
        logger.debug("Created a Callable Statment");

        try {
            callableStmt = getDBTransaction().createCallableStatement(callableStatement, 0);
            logger.debug("Started registering Output Parameters in RMSSUB_XORDER package");
            callableStmt.registerOutParameter(1, Types.VARCHAR);
            callableStmt.registerOutParameter(2, Types.VARCHAR);
            logger.debug("Completed Registration of Output Parameters in RMSSUB_XORDER package");
            logger.debug("Started setting input Parameters in RMSSUB_XORDER package");
            callableStmt.setString(1, "");
            callableStmt.setString(2, "");
            callableStmt.setObject(3, getRibXorderdescRecObject());
            callableStmt.setString(4, "xordermod");
            logger.debug("Completed setting the input parameters in RMSSUB_XORDER package");
            logger.debug("Started execution of the PL/SQL package RMSSUB_XORDER");
            callableStmt.executeUpdate();
            logger.debug("Completed Execution of the PL/SQL package RMSAIASUB_SUPPLIER");
            logger.debug("STATUS : " + callableStmt.getString(1));
            logger.debug("ERROR : " + callableStmt.getString(2));
            if (callableStmt.getString(1).equals("S"))
            {
              status = callableStmt.getString(1); 
              /*Refreshing the data in SelectOrderVO1 View object because Purchase order 
               has been cancelled and LOV data has to be refreshed*/
              getSelectOrderVO1().executeQuery();
            }
            else {
                   status = callableStmt.getString(2);   
                 }

            logger.debug("Committing the DB transaction to cancel the order");
            getDBTransaction().commit();
            logger.debug("DB Transaction is committed ");
            callableStmt.close();
            return status;
        } catch (SQLException e) {
            e.printStackTrace();
            logger.error("SQLException in callable statement execution", e);
            throw new JboException(e);
        }
    }

    /**
     * Method getRibXorderdescRecObject creates an instance of RibXorderdescRec object to be used
     * in the {@link #cancelOrder} method of this class
     * @return Object Returns an instance of RibXorderdescRec
     */
    private Object getRibXorderdescRecObject() {
        BigDecimal dept = null;
        logger.debug("Creating an instance of RibXorderdescRec");
        RibXorderdescRec xorderdescRec = new RibXorderdescRec();
        logger.debug("Created instance of RibXorderdescRec");
        try {
            logger.debug("Fetching values for RibXorderdescRec :");
            Object orderNo = getOrderLocVO1().getCurrentRow().getAttribute("OrderNo");
            logger.debug("Fetched Order number : " + orderNo);

            Object supplierNo = getOrderLocVO1().getCurrentRow().getAttribute("Supplier");
            logger.debug("Fetched supplier : " + supplierNo);

            Object deptNo = getOrderLocVO1().getCurrentRow().getAttribute("Dept");
            logger.debug("Fetched department number : " + deptNo);
            if (deptNo != null) {
                dept = new BigDecimal(deptNo.toString());
            }

            Object terms = getOrderLocVO1().getCurrentRow().getAttribute("Terms");
            logger.debug("Fetched terms : " + terms);
        
            logger.debug("Setting the values in RibXorderdescRec");
            xorderdescRec.setOrderNo(orderNo.toString());
            logger.debug("Setting the orderNo in RibXorderdescRec");
            xorderdescRec.setSupplier(supplierNo.toString());
            logger.debug("Setting the supplierNo in RibXorderdescRec");
            xorderdescRec.setTerms(terms.toString());
            logger.debug("Setting the terms in RibXorderdescRec");
            xorderdescRec.setDept(dept);
            logger.debug("Setting the dept in RibXorderdescRec");
            xorderdescRec.setStatus("C");
            logger.debug("Setting the status in RibXorderdescRec");
            xorderdescRec.setXorderdtlTbl(getXorderdtlTbl());
            logger.debug("Setting the XorderdtlTbl in RibXorderdescRec");
            logger.debug("Values successfully set in RibXorderdescRec");
        }  catch (Exception e) {
            e.printStackTrace();
            logger.error("Error in creating RibXorderdescRec RIB object: ", e);
            throw new JboException("Error in creating RibXorderdescRec RIB Object" + e);
        }
        logger.debug("Instance of RibXorderdescRec successfully created");
        return xorderdescRec;
    }

    /**
     * Method getXorderdtlTbl creates and populates an instance of RibXorderdtlTbl object
     * to be used to populate the RibXorderdescRec object created in
     * {@link #getRibXorderdescRecObject} method.
     * @return RibXorderdtlTbl Returns an instance of RibXorderdtlTbl object
     */
    private RibXorderdtlTbl getXorderdtlTbl() {
        logger.debug("Creating an instance of RibXorderdtlTbl");
        RibXorderdtlTbl xorderdtlTbl = new RibXorderdtlTbl();
        logger.debug("Created instance of RibXorderdtlTbl");
        try {
            logger.debug("Setting the values in RibXorderdtlTbl");
            xorderdtlTbl.setArray(getXorderdtlRec());
            logger.debug("Values successfully set in RibXorderdtlTbl");
        } catch (SQLException e) {
            e.printStackTrace();
            logger.error("Error in creating RibXorderdtlTbl RIB object: ", e);
            throw new JboException("Error in creating RibXorderdtlTbl RIB Object" +
                                   e);
        }
        logger.debug("Instance of RibXorderdtlTbl successfully created");
        return xorderdtlTbl;
    }


    /**
     * Method getXorderdtlRec creates and returns an Array of RibXorderdtlRec object
     * to be used to populate the RibXorderdtlTbl object created in {@link #getXorderdtlTbl}
     * method
     * @return RibXorderdtlRec[] Returns an array of RibXorderdtlRec object
     */
    private RibXorderdtlRec[] getXorderdtlRec() {
        logger.debug("Creating an instance of RibXorderdtlRec");
        RibXorderdtlRec xorderdtlRec = new RibXorderdtlRec();
        logger.debug("Created instance of RibXorderdtlRec");
        logger.debug("Started fetching values from the Order Details View object to populate" +
                     "RibXorderdtlRec instance");

        try {
            logger.debug("Fetching values for RibXorderdtlRec :");
            Object location =
                getOrderLocVO1().getCurrentRow().getAttribute("Location");
            logger.debug("Fetched Location : " + location);
            Object itemId =
                getOrderLocVO1().getCurrentRow().getAttribute("Item");
            logger.debug("Fetched Item  : " + itemId);
            Object locType =
                getOrderLocVO1().getCurrentRow().getAttribute("LocType");
            logger.debug("Fetched Location Type : " + locType);
            Object qtyOrdered =
                getOrderLocVO1().getCurrentRow().getAttribute("QtyOrdered");
            logger.debug("Fetched Quantity Ordered : " + qtyOrdered);

            logger.debug("Setting the values in RibXorderdtlRec");
            xorderdtlRec.setItem(itemId.toString()); //String item,
            logger.debug("Setting the itemId in RibXorderdtlRec");
            xorderdtlRec.setLocation(new BigDecimal(location.toString())); //java.math.BigDecimal location,
            logger.debug("Setting the location in RibXorderdtlRec");
            xorderdtlRec.setQtyOrdered(new BigDecimal(qtyOrdered.toString())); //java.math.BigDecimal qtyOrdered,
            logger.debug("Setting the qtyOrdered in RibXorderdtlRec");
            xorderdtlRec.setLocationType(locType.toString()); //String locationType,
            logger.debug("Setting the locType in RibXorderdtlRec");
            logger.debug("Values set succesfully in RibXorderdtlRec instance");
        } catch (SQLException e) {
            e.printStackTrace();
            logger.error("Error in creating RibXorderdtlRec RIB object: ", e);
            throw new JboException("Error in creating RibXorderdtlRec RIB Object" +
                                   e);
        }
        logger.debug("Instance of RibXorderdtlRec created successfully");
        return new RibXorderdtlRec[] { xorderdtlRec };
    }
}
