package oracle.retail.microapps.order.cancel.am.applicationModule;

import oracle.jbo.JboException;
import oracle.jbo.Row;
import oracle.jbo.ViewObject;
import oracle.jbo.domain.Number;

import oracle.retail.microapps.order.cancel.am.common.CancelOrder;

import org.apache.log4j.Logger;

import org.junit.*;
import static org.junit.Assert.*;

public class CancelOrderAMTest {
    private CancelOrder _amImpl =
        (CancelOrder)CancelOrderAMFixture.getInstance().getApplicationModule();
    Logger logger = Logger.getLogger(CancelOrderAMTest.class);

    public CancelOrderAMTest() {
    }

    @Before
    public void setUp() {
    }

    @After
    public void tearDown() {
    }

    /**
     * This method is to test the cancelOrder() method to cancel
     * valid Purchase Order Number. This means that order, which exists
     * in database and can be cancelled.
     * Expected Result : Order will be cancelled successfully.
     */
    @Test
    public void testCancelOrderWithValidOrderNumber() {
        logger.debug("Executing testCancelOrderWithValidOrderNumber() test case");
        ViewObject view = _amImpl.findViewObject("OrderLocVO1");
        logger.debug("OrderLoc view object fetched");
        view.setNamedWhereClauseParam("order_no", "62308");
        logger.debug("order_no bind variable set");
        view.executeQuery();
        Row r = view.first();
        String status = _amImpl.cancelOrder();
        logger.debug("cancelOrder method executed and status retrieved ");
        assertTrue(status.equals("S"));
        logger.debug("Execution of  testCancelOrderWithValidOrderNumber() test case completed");
    }

    /**
     * This method is to test the cancelOrder() method to cancel invalid
     * Purchase Order Number which does not exist in database
     * Expected Result : JboException
     */
    @Test(expected = JboException.class)
    public void testCancelOrderOrderNumberNotExists() {
        logger.debug("Executing testCancelOrderWithLargeOrderNumber() test case");
        ViewObject view = _amImpl.findViewObject("OrderLocVO1");
        logger.debug("OrderLoc view object fetched");
        view.setNamedWhereClauseParam("order_no", "999999");
        logger.debug("order_no bind variable set");
        view.executeQuery();
        Row r = view.first();
        logger.debug("Expecting exception in testCancelOrderWithLargeOrderNumber() method");
        String status = _amImpl.cancelOrder();
    }
}
