package oracle.retail.microapps.map.managed.helper;

import java.util.ArrayList;

import java.util.HashMap;
import java.util.List;

import oracle.adf.model.binding.DCIteratorBinding;
import oracle.adf.view.faces.bi.model.GeoMapDataModel;

import oracle.adf.view.faces.bi.model.GeoObject;
import oracle.adf.view.faces.bi.model.GeoRowObject;

import oracle.jbo.Row;
import oracle.jbo.RowSetIterator;
import oracle.jbo.domain.Number;

import oracle.retail.merch20.common.ADFUtils;

import oracle.retail.microapps.map.config.MapConfig;

import org.apache.log4j.Logger;

public class MapModelDispatcher {
    
    private static Logger logger = Logger.getLogger(MapModelDispatcher.class);
    
    public MapModelDispatcher() {
        super();
    }
    
    /**
     * Create GeoMapDataModel for a theme. Themes types currently supported are POINT_THEME,
     * PIE_THEME, BAR_THEME. As an argument, this method takes the configuration Object
     * for the Theme, which contains all configured details for the theme.  
     * @param forTheme
     * @return GeoMapDataModel
     */
    public static GeoMapDataModel dispatchModel(MapConfig.Themes.Theme forTheme){
        //TODO if the types of theme becomes too large, implement factory pattern here...
        DCIteratorBinding dcib = getIterator(forTheme);
        if(forTheme.getType().equalsIgnoreCase("POINT_THEME"))
            return getPointThemeDataModel(forTheme, dcib);
        else if(forTheme.getType().equalsIgnoreCase("PIE_THEME"))
            return getBarOrPieThemeDataModel(forTheme, dcib);
        else if(forTheme.getType().equalsIgnoreCase("BAR_THEME"))
            return getBarOrPieThemeDataModel(forTheme, dcib);
        else
            return null;
    }
    
    /**
     * This is a helper method, finds iterator for the theme passed as argument,
     * executes the query and returns associated DCIteratorBinding.
     * @param forTheme
     * @return
     */
    private static DCIteratorBinding getIterator(MapConfig.Themes.Theme forTheme){
        //getting configured iterator name for the theme
        String viewObjectIter = forTheme.getIteratorName();
        //executing iterator
        DCIteratorBinding dcib = ADFUtils.findIterator(viewObjectIter);
        try{
            logger.debug("Executing iterator "+viewObjectIter);
           dcib.executeQuery();
        } catch(Exception e) {
            logger.debug("Exception while accessing iterator "+viewObjectIter);
            logger.debug(e);
            return null;
        }
        return dcib;
    }
    
   /**
     * This is a helper method which creates GeoMapDataModel for a POINT_THEME.
     * This method uses the iterator configured for the theme to get the point data.
     * It creates the map data model from this data as is required for a POINT_THEME.
     * You may refer to the java doc for GeoMapDataModel for more details.
     * @param forTheme
     * @param dcib
     * @return GeoMapDataModel for POINT_THEME
    */
    private static GeoMapDataModel getPointThemeDataModel(MapConfig.Themes.Theme forTheme, DCIteratorBinding dcib){
        //TODO if logic for creating model becomes complex.. consider Template pattern here...
        logger.debug("Getting Point theme data for theme "+forTheme.getName());
        //determining mapping from config
        //spatial mapping configuration ...
        String iterColumnForLatitude = forTheme.getSpatialDataMapping().getLatitude();
        String iterColumnForLongitude = forTheme.getSpatialDataMapping().getLongitude();
        String iterColumnForLocationLabel = forTheme.getSpatialDataMapping().getLabel();
        String iterColumnForKey = forTheme.getSpatialDataMapping().getKey();
        //business data mapping configuration ...
        String iterColumnForData = forTheme.getBusinessDataMapping().getData().get(0).getIterAttr();
        String dataLabel = forTheme.getBusinessDataMapping().getData().get(0).getLabel();
        String dataFormat = forTheme.getBusinessDataMapping().getData().get(0).getFormat();
        //iterating the rows returned from iterator
        ArrayList geoRowArray = new ArrayList();
        int rowIndex = 0;
        RowSetIterator iterator = dcib.getRowSetIterator();
        while(iterator.hasNext()) {
            Row r = iterator.next();
            rowIndex++;
            //Spatial data mapping
            Number latitude = (Number)r.getAttribute(iterColumnForLatitude);
            Number longitude = (Number)r.getAttribute(iterColumnForLongitude);
            String locationLabel = r.getAttribute(iterColumnForLocationLabel).toString();
            String rowKey = r.getAttribute(iterColumnForKey).toString();
            //Business data mapping
            Number data = (Number)r.getAttribute(iterColumnForData);
            GeoObject key = new GeoObject(rowKey, 
                                          locationLabel, 
                                          new Double(longitude.doubleValue()), 
                                          new Double(latitude.doubleValue()));
            GeoRowObject geoRow = new GeoRowObject(key, new String[]{"Selected Item"}, 
                                                   new String[]{dataLabel}, 
                                                   new Double[]{data.doubleValue()}, 
                                                   new String[]{data.toString()}); //TODO formatting
            geoRowArray.add(geoRow);
        }
        // Populate a GeoMapDataModel with the theme data
        return createDataModel(geoRowArray);
    }
    
    /**
      * This is a helper method which creates GeoMapDataModel for a BAR_THEME or PIE_THEME.
      * This method uses the iterator configured for the theme to get the point data.
      * It creates the map data model from this data as is required for a BAR/PIE_THEME.
      * You may refer to the java doc for GeoMapDataModel for more details.
      * @param forTheme
      * @param dcib
      * @return GeoMapDataModel for Bar or PIE theme
      */
    private static GeoMapDataModel getBarOrPieThemeDataModel(MapConfig.Themes.Theme forTheme, DCIteratorBinding dcib){
        logger.debug("Getting Bar/Pie theme data for theme "+forTheme.getName());
        //determining mapping from config
        //spatial mapping configuration ...
        String iterColumnForLocation = forTheme.getSpatialDataMapping().getLabel();
        String iterColumnForKey = forTheme.getSpatialDataMapping().getKey();
        //business data configuration ...
        List<MapConfig.Themes.Theme.BusinessDataMapping.Data> dataList = forTheme.getBusinessDataMapping().getData();
        int dataListSize = dataList.size();
        String[] dataId = null;
        String[] dataLabel = null;
        Double[] dataValue = null;
        String[] formattedDataValue = null; //TODO formatting
        //iterating the rows returned from iterator
        ArrayList geoRowArray = new ArrayList();
        int rowIndex = 0;
        RowSetIterator iterator = dcib.getRowSetIterator();
        while(iterator.hasNext()) {
            Row r = iterator.next();
            rowIndex++;
            dataId = new String[dataListSize];
            dataLabel = new String[dataListSize];
            dataValue = new Double[dataListSize];
            formattedDataValue = new String[dataListSize]; //TODO formatting
            int dataListIndex = 0;
            for(MapConfig.Themes.Theme.BusinessDataMapping.Data dataPoint: dataList){
                //business data mapping...
                String iterColumnForDataMapp = dataPoint.getIterAttr();
                dataValue[dataListIndex] =  ((Number)r.getAttribute(iterColumnForDataMapp)).doubleValue();
                String dataPointLabel = forTheme.getBusinessDataMapping().getData().get(dataListIndex).getLabel();
                dataLabel[dataListIndex] = dataPointLabel;
                String dataPointFormat = forTheme.getBusinessDataMapping().getData().get(dataListIndex).getFormat();
                formattedDataValue[dataListIndex] = dataValue[dataListIndex].toString(); 
                dataId[dataListIndex] = (rowIndex+""+dataListIndex);
                dataListIndex++;
            }
            //Spatial data mapping
            String locationLabel = r.getAttribute(iterColumnForLocation).toString();
            String key = r.getAttribute(iterColumnForKey).toString();
            GeoObject geoRowkey = new GeoObject(key, 
                                            locationLabel, 
                                            locationLabel);
            GeoRowObject geoRow = new GeoRowObject(geoRowkey, dataId, 
                                                     dataLabel, 
                                                     dataValue, 
                                                     formattedDataValue); //TODO formatting
            geoRowArray.add(geoRow);
        }
        // Populate a GeoMapDataModel with the theme data
        return createDataModel(geoRowArray);
    }
    
    /**
     * This is a helper method for creating GeoMapDataModel from a list of GeoRowObject.
     * @param geoRowList
     * @return GeoMapDataModel
     */
    private static GeoMapDataModel createDataModel(ArrayList geoRowList){
        GeoMapDataModel model = new GeoMapDataModel();
        model.setWrappedData(geoRowList);
        return model;
    }
}
