package oracle.retail.microapps.order.create.managed;

import oracle.adf.model.binding.DCIteratorBinding;
import oracle.adf.share.ADFContext;

import oracle.jbo.Row;

import oracle.retail.merch20.common.ADFUtils;

import oracle.retail.microapps.order.create.backing.CreateOrder;

import org.apache.log4j.Logger;

public class Order {
    
    //getting logger instance
    private Logger logger = Logger.getLogger(Order.class);
    //input params accepted by the createOrder view activity
    private String locationInputParam; //order location
    private String locationTypeInputParam; //order location type (Store or warehouse)
    private String supplierInputParam; // supplier
    private String itemInputParam; // ordered Item 
    
    public Order() {
        super();
    }

    public void setLocationInputParam(String locationInputParam) {
        this.locationInputParam = locationInputParam;
        logger.debug("In OrderBean : Location parameter passed is: " + locationInputParam);
        DCIteratorBinding dcib = null;
        Row rw = null;
        if(locationInputParam != null){
            dcib = ADFUtils.findIterator("LocationVO1Iterator1");
            dcib.setCurrentRowWithKeyValue(locationInputParam);
            rw = dcib.getCurrentRow();
            logger.debug("Current row is : "+rw);
        }
        if (rw != null){
            //location passed is valid
            //Fetching all the variables from location current row
            Object locationIdValue = rw.getAttribute("Location");
            Object locationTypeValue = rw.getAttribute("Loctype");
            Object Orgunitid = rw.getAttribute("Orgunitidb");
            logger.debug("location id fetched from location vo is " + locationIdValue);
            logger.debug("orgUnitID fetched from location vo is " + Orgunitid);
            logger.debug("location type value fetched from location is " + locationTypeValue);
            //setting view scope variables for queries
            ADFContext.getCurrent().getViewScope().put("passedParamLoc", locationIdValue.toString());
            ADFContext.getCurrent().getViewScope().put("passedParamLocType", locationTypeValue.toString());
            ADFContext.getCurrent().getViewScope().put("orgUnitID", Orgunitid);
        } else {
            //if location passed is not present
            logger.debug("Invalid location passed.");
            ADFContext.getCurrent().getViewScope().put("passedParamLoc", null);
            ADFContext.getCurrent().getViewScope().put("passedParamLocType", null);
            ADFContext.getCurrent().getViewScope().put("orgUnitID", null);
        }
    }

    public String getLocationInputParam() {
        return locationInputParam;
    }
    
    public void setLocationTypeInputParam(String locationTypeInputParam) {
        this.locationTypeInputParam = locationTypeInputParam;
        if(locationTypeInputParam != null){
            logger.debug("In OrderBean : Location Type parameter passed is: " + locationTypeInputParam);
            ADFContext.getCurrent().getViewScope().put("passedParamLocType", locationTypeInputParam.toString());
        }
    }
  
    public String getLocationTypeInputParam() {
        return locationTypeInputParam;
    }

    public void setSupplierInputParam(String supplierInputParam) {
        this.supplierInputParam = supplierInputParam;
        logger.debug("In OrderBean : Supplier parameter passed is: " + supplierInputParam);
        DCIteratorBinding dcib = null;
        Row rw = null;
        if(supplierInputParam != null){
            dcib = ADFUtils.findIterator("VSupMsobYVO1Iterator");
            rw = dcib.getCurrentRow();
        }
        if(rw != null){
            //supplier passed is valid
            //setting view scope variables for queries
            ADFContext.getCurrent().getViewScope().put("supplierIDBindValue", supplierInputParam);
        } else {
            logger.debug("Invalid supplier passed.");
            ADFContext.getCurrent().getViewScope().put("supplierIDBindValue", null);
        }
    }

    public String getSupplierInputParam() {
        return supplierInputParam;
    }

    public void setItemInputParam(String itemInputParam) {
        this.itemInputParam = itemInputParam;
        logger.debug("In OrderBean : Item parameter passed is : "+itemInputParam);
        DCIteratorBinding dcib = null;
        Row rw = null;
        if(itemInputParam != null){
            dcib = ADFUtils.findIterator("ItemsVO1Iterator"); 
            rw = dcib.getCurrentRow();
        }
        if(rw != null){
            //item passed is valid
            //create a row in ordered item view object
            CreateOrder.populateOrderedItemVO(rw);
            ADFContext.getCurrent().getViewScope().put("passedItem", itemInputParam);
        } else {
            logger.debug("Invalid item passed.");
            ADFContext.getCurrent().getViewScope().put("passedItem", null);
        }
    }

    public String getItemInputParam() {
        return itemInputParam;
    }
}
